"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const controller_1 = require("@curveball/controller");
const http_errors_1 = require("@curveball/http-errors");
const querystring = require("querystring");
const service_1 = require("../../log/service");
const types_1 = require("../../log/types");
const webAuthnService = require("../../mfa/webauthn/service");
const server_settings_1 = require("../../server-settings");
const principalService = require("../../principal/service");
const userService = require("../../user/service");
const utilities_1 = require("../utilities");
const html_1 = require("../formats/html");
class LoginController extends controller_1.default {
    async get(ctx) {
        const firstRun = !(await principalService.hasPrincipals());
        if (firstRun) {
            ctx.redirect(302, '/register');
            return;
        }
        ctx.response.type = 'text/html';
        ctx.response.body = html_1.loginForm(ctx.query.msg, ctx.query.error, {
            continue: ctx.query.continue,
        }, await server_settings_1.getSetting('registration.enabled'));
    }
    async post(ctx) {
        let user;
        try {
            user = await principalService.findByIdentity('mailto:' + ctx.request.body.userName);
        }
        catch (err) {
            if (err instanceof http_errors_1.NotFound) {
                service_1.default(types_1.EventType.loginFailed, ctx);
                return this.redirectToLogin(ctx, '', 'Incorrect username or password');
            }
            else {
                throw err;
            }
        }
        if (!await userService.validatePassword(user, ctx.request.body.password)) {
            service_1.default(types_1.EventType.loginFailed, ctx.ip(), user.id);
            return this.redirectToLogin(ctx, '', 'Incorrect username or password');
        }
        if (!user.active) {
            service_1.default(types_1.EventType.loginFailedInactive, ctx.ip(), user.id, ctx.request.headers.get('User-Agent'));
            return this.redirectToLogin(ctx, '', 'This account is inactive. Please contact Admin');
        }
        if (await this.shouldMfaRedirect(ctx, user)) {
            return;
        }
        ctx.session = {
            user: user,
        };
        service_1.default(types_1.EventType.loginSuccess, ctx);
        ctx.status = 303;
        if (ctx.request.body.continue) {
            ctx.response.headers.set('Location', ctx.request.body.continue);
            return;
        }
        ctx.response.redirect(303, server_settings_1.getSetting('login.defaultRedirect', '/'));
    }
    redirectToLogin(ctx, msg, error) {
        var _a;
        const params = { msg, error };
        if ((_a = ctx.request.body) === null || _a === void 0 ? void 0 : _a.continue) {
            params['continue'] = ctx.request.body.continue;
        }
        ctx.response.status = 303;
        ctx.response.headers.set('Location', '/login?' + querystring.stringify(params));
    }
    async shouldMfaRedirect(ctx, user) {
        if (!this.isMfaEnabled()) {
            return false;
        }
        if (await this.shouldUseTotp(ctx, user)) {
            return true;
        }
        if (await this.shouldUseWebauthn(ctx, user)) {
            return true;
        }
        return false;
    }
    async shouldUseTotp(ctx, user) {
        if (server_settings_1.getSetting('totp') !== 'disabled') {
            if (await userService.hasTotp(user)) {
                if (ctx.request.body.continue && !utilities_1.isValidRedirect(ctx.request.body.continue)) {
                    this.redirectToLogin(ctx, '', 'Invalid continue URL provided');
                    return true;
                }
                const mfaSession = {
                    user,
                    mfaType: 'totp'
                };
                ctx.session = {
                    mfa: mfaSession,
                };
                this.redirectToMfa(ctx, ctx.request.body.continue);
                return true;
            }
            if (server_settings_1.getSetting('totp') === 'required') {
                this.redirectToLogin(ctx, '', 'The system administrator has made TOTP tokens mandatory, but this user did not have a TOTP configured. Login is disabled');
                return true;
            }
        }
        return false;
    }
    async shouldUseWebauthn(ctx, user) {
        if (server_settings_1.getSetting('webauthn') !== 'disabled') {
            if (await webAuthnService.hasWebauthn(user)) {
                if (ctx.request.body.continue && !utilities_1.isValidRedirect(ctx.request.body.continue)) {
                    this.redirectToLogin(ctx, '', 'Invalid continue URL provided');
                    return true;
                }
                const mfaSession = {
                    user,
                    mfaType: 'webauthn'
                };
                ctx.session = {
                    mfa: mfaSession,
                };
                this.redirectToMfa(ctx, ctx.request.body.continue);
                return true;
            }
            if (server_settings_1.getSetting('webauthn') === 'required') {
                this.redirectToLogin(ctx, '', 'The system administrator has made Webauthn mandatory, but this user did not have a Webauthn device configured. Login is disabled');
                return true;
            }
        }
        return false;
    }
    isMfaEnabled() {
        return server_settings_1.getSetting('totp') !== 'disabled' || server_settings_1.getSetting('webauthn') !== 'disabled';
    }
    redirectToMfa(ctx, redirectUrl) {
        ctx.response.status = 303;
        if (redirectUrl) {
            ctx.response.headers.set('Location', '/login/mfa?' + querystring.stringify({ 'continue': redirectUrl }));
        }
        else {
            ctx.response.headers.set('Location', '/login/mfa');
        }
    }
}
exports.default = new LoginController();
//# sourceMappingURL=login.js.map