"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateSecret = exports.create = exports.getOAuth2ClientFromBody = exports.getOAuth2ClientFromBasicAuth = exports.findByApp = exports.findById = exports.findByClientId = void 0;
const bcrypt = require("bcrypt");
const principalService = require("../principal/service");
const database_1 = require("../database");
const http_errors_1 = require("@curveball/http-errors");
const errors_1 = require("../oauth2/errors");
const parse_basic_auth_1 = require("./parse-basic-auth");
async function findByClientId(clientId) {
    const query = 'SELECT id, client_id, client_secret, user_id, allowed_grant_types FROM oauth2_clients WHERE client_id = ?';
    const result = await database_1.default.query(query, [clientId]);
    if (!result[0].length) {
        throw new http_errors_1.NotFound('OAuth2 client_id not recognized');
    }
    const record = result[0][0];
    const user = await principalService.findActiveById(record.user_id);
    return mapRecordToModel(record, user);
}
exports.findByClientId = findByClientId;
async function findById(id) {
    const query = 'SELECT id, client_id, client_secret, user_id, allowed_grant_types FROM oauth2_clients WHERE id = ?';
    const result = await database_1.default.query(query, [id]);
    if (!result[0].length) {
        throw new http_errors_1.NotFound('OAuth2 id not recognized');
    }
    const record = result[0][0];
    const user = await principalService.findActiveById(record.user_id);
    return mapRecordToModel(record, user);
}
exports.findById = findById;
async function findByApp(user) {
    const query = 'SELECT id, client_id, client_secret, user_id, allowed_grant_types FROM oauth2_clients WHERE user_id = ?';
    const result = await database_1.default.query(query, [user.id]);
    return result[0].map((record) => mapRecordToModel(record, user));
}
exports.findByApp = findByApp;
function mapRecordToModel(record, app) {
    return {
        id: record.id,
        clientId: record.client_id,
        clientSecret: record.client_secret.toString('utf-8'),
        app,
        allowedGrantTypes: record.allowed_grant_types.split(' '),
    };
}
async function getOAuth2ClientFromBasicAuth(ctx) {
    let oauth2Client;
    const basicAuth = parse_basic_auth_1.default(ctx);
    if (!basicAuth) {
        throw new http_errors_1.Unauthorized('Invalid Basic Auth header');
    }
    try {
        oauth2Client = await findByClientId(basicAuth[0]);
    }
    catch (e) {
        if (e instanceof http_errors_1.NotFound) {
            throw new http_errors_1.Unauthorized('Client id or secret incorrect', 'Basic');
        }
        else {
            // Rethrow
            throw e;
        }
    }
    if (!await validateSecret(oauth2Client, basicAuth[1])) {
        throw new http_errors_1.Unauthorized('Client id or secret incorrect', 'Basic');
    }
    return oauth2Client;
}
exports.getOAuth2ClientFromBasicAuth = getOAuth2ClientFromBasicAuth;
async function getOAuth2ClientFromBody(ctx) {
    if (!ctx.request.body.client_id) {
        throw new errors_1.InvalidRequest('The "client_id" property is required');
    }
    try {
        return await findByClientId(ctx.request.body.client_id);
    }
    catch (e) {
        if (e instanceof http_errors_1.NotFound) {
            throw new http_errors_1.Unauthorized('Client id unknown', 'Basic');
        }
        else {
            // Rethrow
            throw e;
        }
    }
}
exports.getOAuth2ClientFromBody = getOAuth2ClientFromBody;
async function create(client, redirectUris) {
    const query = 'INSERT INTO oauth2_clients SET ?';
    const params = {
        client_id: client.clientId,
        client_secret: client.clientSecret,
        user_id: client.app.id,
        allowed_grant_types: client.allowedGrantTypes.join(' '),
    };
    const result = await database_1.default.query(query, [params]);
    const realClient = {
        ...client,
        id: result[0].insertId
    };
    for (const uri of redirectUris) {
        await database_1.default.query('INSERT INTO oauth2_redirect_uris SET ?', [{ oauth2_client_id: realClient.id, uri }]);
    }
    return realClient;
}
exports.create = create;
async function validateSecret(oauth2Client, secret) {
    return await bcrypt.compare(secret, oauth2Client.clientSecret);
}
exports.validateSecret = validateSecret;
//# sourceMappingURL=service.js.map