"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const controller_1 = require("@curveball/controller");
const service_1 = require("../../log/service");
const types_1 = require("../../log/types");
const principalService = require("../../principal/service");
const userService = require("../../user/service");
const errors_1 = require("../errors");
const oauth2Service = require("../service");
const service_2 = require("../../oauth2-client/service");
class TokenController extends controller_1.default {
    async post(ctx) {
        this.sendCORSHeaders(ctx);
        const supportedGrantTypes = ['client_credentials', 'authorization_code', 'refresh_token', 'password'];
        const grantType = ctx.request.body.grant_type;
        if (!supportedGrantTypes.includes(grantType)) {
            throw new errors_1.UnsupportedGrantType('The "grant_type" must be one of ' + supportedGrantTypes.join(', '));
        }
        let oauth2Client;
        switch (grantType) {
            case 'authorization_code':
                oauth2Client = await service_2.getOAuth2ClientFromBody(ctx);
                break;
            case 'refresh_token':
                if (ctx.request.headers.has('Authorization')) {
                    oauth2Client = await service_2.getOAuth2ClientFromBasicAuth(ctx);
                }
                else {
                    oauth2Client = await service_2.getOAuth2ClientFromBody(ctx);
                }
                break;
            default:
                oauth2Client = await service_2.getOAuth2ClientFromBasicAuth(ctx);
                break;
        }
        if (!oauth2Client.allowedGrantTypes.includes(grantType)) {
            throw new errors_1.UnsupportedGrantType('The current client is not allowed to use the ' + grantType + ' grant_type');
        }
        switch (grantType) {
            case 'authorization_code':
                return this.authorizationCode(oauth2Client, ctx);
            case 'client_credentials':
                return this.clientCredentials(oauth2Client, ctx);
            case 'password':
                return this.password(oauth2Client, ctx);
            case 'refresh_token':
                return this.refreshToken(oauth2Client, ctx);
        }
    }
    async clientCredentials(oauth2Client, ctx) {
        const token = await oauth2Service.generateTokenForClient(oauth2Client);
        ctx.response.type = 'application/json';
        ctx.response.body = {
            access_token: token.accessToken,
            token_type: token.tokenType,
            expires_in: token.accessTokenExpires - Math.round(Date.now() / 1000),
        };
    }
    async authorizationCode(oauth2Client, ctx) {
        if (!ctx.request.body.code) {
            throw new errors_1.InvalidRequest('The "code" property is required');
        }
        if (!ctx.request.body.redirect_uri) {
            throw new errors_1.InvalidRequest('The "redirect_uri" property is required');
        }
        if (!await oauth2Service.validateRedirectUri(oauth2Client, ctx.request.body.redirect_uri)) {
            service_1.default(types_1.EventType.oauth2BadRedirect, ctx);
            throw new errors_1.InvalidRequest('This value for "redirect_uri" is not recognized.');
        }
        const token = await oauth2Service.generateTokenFromCode(oauth2Client, ctx.request.body.code, ctx.request.body.code_verifier);
        ctx.response.type = 'application/json';
        ctx.response.body = {
            access_token: token.accessToken,
            token_type: token.tokenType,
            expires_in: token.accessTokenExpires - Math.round(Date.now() / 1000),
            refresh_token: token.refreshToken,
        };
    }
    async password(oauth2Client, ctx) {
        let user;
        try {
            user = await principalService.findByIdentity('mailto:' + ctx.request.body.username);
            if (user.type !== 'user') {
                throw new errors_1.InvalidRequest('The "password" grant type is only valid for users');
            }
        }
        catch (err) {
            throw new errors_1.InvalidGrant('Unknown username or password');
        }
        if (!await userService.validatePassword(user, ctx.request.body.password)) {
            service_1.default(types_1.EventType.loginFailed, ctx.ip(), user.id);
            throw new errors_1.InvalidGrant('Unknown username or password');
        }
        if (!user.active) {
            service_1.default(types_1.EventType.loginFailedInactive, ctx.ip(), user.id, ctx.request.headers.get('User-Agent'));
            throw new errors_1.InvalidGrant('User Inactive');
        }
        service_1.default(types_1.EventType.loginSuccess, ctx);
        const token = await oauth2Service.generateTokenForUser(oauth2Client, user);
        ctx.response.body = {
            access_token: token.accessToken,
            token_type: token.tokenType,
            expires_in: token.accessTokenExpires - Math.round(Date.now() / 1000),
            refresh_token: token.refreshToken,
        };
    }
    async refreshToken(oauth2Client, ctx) {
        if (!ctx.request.body.refresh_token) {
            throw new errors_1.InvalidRequest('The "refresh_token" property is required');
        }
        const token = await oauth2Service.generateTokenFromRefreshToken(oauth2Client, ctx.request.body.refresh_token);
        ctx.response.body = {
            access_token: token.accessToken,
            token_type: token.tokenType,
            expires_in: token.accessTokenExpires - Math.round(Date.now() / 1000),
            refresh_token: token.refreshToken,
        };
    }
    async options(ctx) {
        ctx.status = 200;
        this.sendCORSHeaders(ctx);
    }
    sendCORSHeaders(ctx) {
        ctx.response.headers.set('Access-Control-Allow-Origin', '*');
    }
}
exports.default = new TokenController();
//# sourceMappingURL=token.js.map