"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.invalidateTokensByBrowserSessionId = exports.getTokenByRefreshToken = exports.getTokenByAccessToken = exports.generateCodeForUser = exports.revokeToken = exports.revokeByAccessRefreshToken = exports.generateTokenFromRefreshToken = exports.validatePKCE = exports.generateTokenFromCode = exports.generateTokenForClient = exports.generateTokenForUserNoClient = exports.generateTokenForUser = exports.getActiveTokens = exports.addRedirectUris = exports.requireRedirectUri = exports.validateRedirectUri = exports.getRedirectUris = void 0;
const http_errors_1 = require("@curveball/http-errors");
const crypto = require("crypto");
const database_1 = require("../database");
const server_settings_1 = require("../server-settings");
const principalService = require("../principal/service");
const errors_1 = require("./errors");
const crypto_1 = require("../crypto");
const jwt_1 = require("./jwt");
async function getRedirectUris(client) {
    const query = 'SELECT uri FROM oauth2_redirect_uris WHERE oauth2_client_id = ?';
    const result = await database_1.default.query(query, [client.id]);
    return result[0].map((record) => record.uri);
}
exports.getRedirectUris = getRedirectUris;
async function validateRedirectUri(client, redirectUri) {
    const uris = await getRedirectUris(client);
    return uris.includes(redirectUri);
}
exports.validateRedirectUri = validateRedirectUri;
/**
 * Checks if a redirect_uri is permitted for the client.
 *
 * If not, it will emit an InvalidGrant error
 */
async function requireRedirectUri(client, redirectUrl) {
    const uris = await getRedirectUris(client);
    if (uris.length === 0) {
        throw new errors_1.InvalidGrant('No valid redirect_uri was setup for this OAuth2 client_id');
    }
    if (!uris.includes(redirectUrl)) {
        throw new errors_1.InvalidGrant(`Invalid value for redirect_uri. The redirect_uri you passed (${redirectUrl}) was not in the allowed list of redirect_uris`);
    }
}
exports.requireRedirectUri = requireRedirectUri;
async function addRedirectUris(client, redirectUris) {
    const query = 'INSERT INTO oauth2_redirect_uris SET oauth2_client_id = ?, uri = ?';
    for (const uri of redirectUris) {
        await database_1.default.query(query, [client.id, uri]);
    }
}
exports.addRedirectUris = addRedirectUris;
async function getActiveTokens(user) {
    const query = `
    SELECT
     oauth2_client_id,
     access_token,
     refresh_token,
     user_id,
     access_token_expires,
     refresh_token_expires,
     browser_session_id
    FROM oauth2_tokens
    WHERE 
      user_id = ?
    AND
      refresh_token_expires > UNIX_TIMESTAMP()
  `;
    const result = await database_1.default.query(query, [user.id]);
    return result[0].map((row) => {
        return {
            accessToken: row.access_token,
            refreshToken: row.refresh_token,
            accessTokenExpires: row.access_token_expires,
            refreshTokenExpires: row.refresh_token_expires,
            tokenType: 'bearer',
            user,
            clientId: row.oauth2_client_id,
        };
    });
}
exports.getActiveTokens = getActiveTokens;
/**
 * This function is used for the implicit grant oauth2 flow.
 *
 * This function creates an access token for a specific user.
 */
async function generateTokenForUser(client, user, browserSessionId) {
    if (!user.active) {
        throw new Error('Cannot generate token for inactive user');
    }
    const expirySettings = getTokenExpiry();
    const accessTokenExpires = Math.floor(Date.now() / 1000) + expirySettings.accessToken;
    const refreshTokenExpires = Math.floor(Date.now() / 1000) + expirySettings.refreshToken;
    let accessToken;
    if (process.env.JWT_PRIVATE_KEY) {
        accessToken = await jwt_1.generateJWTAccessToken(user, client, expirySettings.accessToken, []);
    }
    else {
        accessToken = await crypto_1.generateSecretToken();
    }
    const refreshToken = await crypto_1.generateSecretToken();
    const query = 'INSERT INTO oauth2_tokens SET created = UNIX_TIMESTAMP(), ?';
    await database_1.default.query(query, {
        oauth2_client_id: client.id,
        access_token: accessToken,
        refresh_token: refreshToken,
        user_id: user.id,
        access_token_expires: accessTokenExpires,
        refresh_token_expires: refreshTokenExpires,
        browser_session_id: browserSessionId,
    });
    return {
        accessToken,
        refreshToken,
        accessTokenExpires,
        refreshTokenExpires,
        tokenType: 'bearer',
        user,
        clientId: client.id,
    };
}
exports.generateTokenForUser = generateTokenForUser;
/**
 * This function is used to create arbitrary access tokens, by the currently logged in user.
 *
 * There is no specific OAuth2 flow for this.
 */
async function generateTokenForUserNoClient(user) {
    if (!user.active) {
        throw new Error('Cannot generate token for inactive user');
    }
    const accessToken = await crypto_1.generateSecretToken();
    const refreshToken = await crypto_1.generateSecretToken();
    const query = 'INSERT INTO oauth2_tokens SET created = UNIX_TIMESTAMP(), ?';
    const expirySettings = getTokenExpiry();
    const accessTokenExpires = Math.floor(Date.now() / 1000) + expirySettings.accessToken;
    const refreshTokenExpires = Math.floor(Date.now() / 1000) + expirySettings.refreshToken;
    await database_1.default.query(query, {
        oauth2_client_id: 0,
        access_token: accessToken,
        refresh_token: refreshToken,
        user_id: user.id,
        access_token_expires: accessTokenExpires,
        refresh_token_expires: refreshTokenExpires,
        browser_session_id: null,
    });
    return {
        accessToken,
        refreshToken,
        accessTokenExpires,
        refreshTokenExpires,
        tokenType: 'bearer',
        user,
    };
}
exports.generateTokenForUserNoClient = generateTokenForUserNoClient;
/**
 * This function is used for the client_credentials oauth2 flow.
 *
 * In this flow there is not a 3rd party (resource owner). There is simply 2
 * clients talk to each other.
 *
 * The client acts on behalf of itself, not someone else.
 */
async function generateTokenForClient(client) {
    const accessToken = await crypto_1.generateSecretToken();
    const refreshToken = await crypto_1.generateSecretToken();
    const query = 'INSERT INTO oauth2_tokens SET created = UNIX_TIMESTAMP(), ?';
    const expirySettings = getTokenExpiry();
    const accessTokenExpires = Math.floor(Date.now() / 1000) + expirySettings.accessToken;
    const refreshTokenExpires = Math.floor(Date.now() / 1000) + expirySettings.refreshToken;
    await database_1.default.query(query, {
        oauth2_client_id: client.id,
        access_token: accessToken,
        refresh_token: refreshToken,
        user_id: client.app.id,
        access_token_expires: accessTokenExpires,
        refresh_token_expires: refreshTokenExpires,
    });
    return {
        accessToken,
        refreshToken,
        accessTokenExpires,
        refreshTokenExpires,
        tokenType: 'bearer',
        user: client.app,
        clientId: client.id,
    };
}
exports.generateTokenForClient = generateTokenForClient;
/**
 * This function is used for the authorization_code oauth2 flow.
 *
 * In this flow a user first authenticates itself and grants permisssion to
 * the client. After gaining permission the user gets redirected back to the
 * resource owner with a one-time code.
 *
 * The resource owner then exchanges that code for an access and refresh token.
 */
async function generateTokenFromCode(client, code, codeVerifier) {
    const query = 'SELECT * FROM oauth2_codes WHERE code = ?';
    const codeResult = await database_1.default.query(query, [code]);
    if (!codeResult[0].length) {
        throw new errors_1.InvalidRequest('The supplied code was not recognized');
    }
    const codeRecord = codeResult[0][0];
    const expirySettings = getTokenExpiry();
    // Delete immediately.
    await database_1.default.query('DELETE FROM oauth2_codes WHERE id = ?', [codeRecord.id]);
    validatePKCE(codeVerifier, codeRecord.code_challenge, codeRecord.code_challenge_method);
    if (codeRecord.created + expirySettings.code < Math.floor(Date.now() / 1000)) {
        throw new errors_1.InvalidRequest('The supplied code has expired');
    }
    if (codeRecord.client_id !== client.id) {
        throw new errors_1.UnauthorizedClient('The client_id associated with the token did not match with the authenticated client credentials');
    }
    const user = await principalService.findById(codeRecord.user_id);
    return generateTokenForUser(client, user, codeRecord.browser_session_id || undefined);
}
exports.generateTokenFromCode = generateTokenFromCode;
function validatePKCE(codeVerifier, codeChallenge, codeChallengeMethod) {
    if (!codeVerifier) {
        if (!codeChallenge) {
            // This request was not initiated with PKCE support, so ignore the validation
            return;
        }
        else {
            // The authorization request started with PKCE, but the token request did not follow through
            throw new errors_1.InvalidRequest('The code verifier was not supplied');
        }
    }
    // For the plain method, the derived code and the code verifier are the same
    let derivedCodeChallenge = codeVerifier;
    if (codeChallengeMethod === 'S256') {
        derivedCodeChallenge = crypto.createHash('sha256').update(codeVerifier).digest('base64')
            .replace(/\+/g, '-')
            .replace(/\//g, '_')
            .replace(/=/g, '');
    }
    if (codeChallenge !== derivedCodeChallenge) {
        throw new errors_1.InvalidGrant('The code verifier does not match the code challenge');
    }
}
exports.validatePKCE = validatePKCE;
/**
 * This function is used for the 'refresh_token' grant.
 *
 * By specifying a refresh token, a new access/refresh token pair gets
 * returned. This also expires the old token.
 */
async function generateTokenFromRefreshToken(client, refreshToken) {
    let oldToken;
    try {
        oldToken = await getTokenByRefreshToken(refreshToken);
    }
    catch (err) {
        if (err instanceof http_errors_1.NotFound) {
            throw new errors_1.InvalidGrant('The refresh token was not recognized');
        }
        else {
            throw err;
        }
    }
    if (oldToken.clientId !== client.id) {
        throw new errors_1.UnauthorizedClient('The client_id associated with the refresh did not match with the authenticated client credentials');
    }
    await revokeToken(oldToken);
    return generateTokenForUser(client, oldToken.user, oldToken.browserSessionId);
}
exports.generateTokenFromRefreshToken = generateTokenFromRefreshToken;
async function revokeByAccessRefreshToken(client, token) {
    let oauth2Token = null;
    try {
        oauth2Token = await getTokenByAccessToken(token);
    }
    catch (err) {
        if (err instanceof http_errors_1.NotFound) {
            // Swallow since it's okay if the token has been revoked previously or is invalid (Section 2.2 of RFC7009)
        }
        else {
            throw err;
        }
    }
    if (!oauth2Token) {
        try {
            oauth2Token = await getTokenByRefreshToken(token);
        }
        catch (err) {
            if (err instanceof http_errors_1.NotFound) {
                // Swallow since it's okay if the token has been revoked previously or is invalid (Section 2.2 of RFC7009)
            }
            else {
                throw err;
            }
        }
    }
    if (!oauth2Token) {
        return;
    }
    if (oauth2Token.clientId !== client.id) {
        // Treat this as an invalid token and don't do anything
        return;
    }
    await revokeToken(oauth2Token);
}
exports.revokeByAccessRefreshToken = revokeByAccessRefreshToken;
/**
 * Removes a token.
 *
 * This function will not throw an error if the token was deleted before.
 */
async function revokeToken(token) {
    const query = 'DELETE FROM oauth2_tokens WHERE access_token = ?';
    await database_1.default.query(query, [token.accessToken]);
}
exports.revokeToken = revokeToken;
/**
 * This function is used for the authorization_code grant flow.
 *
 * This function creates an code for a user. The code is later exchanged for
 * a oauth2 access token.
 */
async function generateCodeForUser(client, user, codeChallenge, codeChallengeMethod, browserSessionId) {
    const code = await crypto_1.generateSecretToken();
    const query = 'INSERT INTO oauth2_codes SET created = UNIX_TIMESTAMP(), ?';
    await database_1.default.query(query, {
        client_id: client.id,
        user_id: user.id,
        code: code,
        code_challenge: codeChallenge,
        code_challenge_method: codeChallengeMethod,
        browser_session_id: browserSessionId,
    });
    return {
        code: code
    };
}
exports.generateCodeForUser = generateCodeForUser;
/**
 * Returns Token information for an existing Access Token.
 *
 * This effectively gives you all information of an access token if you have
 * just the bearer, and allows you to validate if a bearer token is valid.
 *
 * This function will throw NotFound if the token was not recognized.
 */
async function getTokenByAccessToken(accessToken) {
    const query = `
  SELECT
   oauth2_client_id,
   access_token,
   refresh_token,
   user_id,
   access_token_expires,
   refresh_token_expires,
   browser_session_id
  FROM oauth2_tokens
  WHERE
    access_token = ? AND
    access_token_expires > UNIX_TIMESTAMP()
  `;
    const result = await database_1.default.query(query, [accessToken]);
    if (!result[0].length) {
        throw new http_errors_1.NotFound('Access token not recognized');
    }
    const row = result[0][0];
    const user = await principalService.findActiveById(row.user_id);
    return {
        accessToken: row.access_token,
        refreshToken: row.refresh_token,
        accessTokenExpires: row.access_token_expires,
        refreshTokenExpires: row.refresh_token_expires,
        tokenType: 'bearer',
        user: user,
        clientId: row.oauth2_client_id,
        browserSessionId: row.browser_session_id || undefined,
    };
}
exports.getTokenByAccessToken = getTokenByAccessToken;
/**
 * Returns Token information for an existing Refresh Token.
 *
 * This function will throw NotFound if the token was not recognized.
 */
async function getTokenByRefreshToken(refreshToken) {
    const query = `
  SELECT
   oauth2_client_id,
   access_token,
   refresh_token,
   user_id,
   access_token_expires,
   refresh_token_expires,
   browser_session_id
  FROM oauth2_tokens
  WHERE
    refresh_token = ? AND
    refresh_token_expires > UNIX_TIMESTAMP()
  `;
    const result = await database_1.default.query(query, [refreshToken]);
    if (!result[0].length) {
        throw new http_errors_1.NotFound('Refresh token not recognized');
    }
    const row = result[0][0];
    const user = await principalService.findActiveById(row.user_id);
    return {
        accessToken: row.access_token,
        refreshToken: row.refresh_token,
        accessTokenExpires: row.access_token_expires,
        refreshTokenExpires: row.refresh_token_expires,
        tokenType: 'bearer',
        user: user,
        clientId: row.oauth2_client_id,
        browserSessionId: row.browser_session_id || undefined,
    };
}
exports.getTokenByRefreshToken = getTokenByRefreshToken;
/**
 * Removes all tokens that relate to a specific browser session id.
 *
 * This will cause all access tokens and refresh tokens to be invalidated. Generally
 * used when a user logs out.
 *
 * This doesn't remove all tokens for all sessions, but should remove the tokens that
 * relate to the device the user used to log out.
 */
async function invalidateTokensByBrowserSessionId(browserSessionId) {
    await database_1.default.query('DELETE FROM oauth2_codes WHERE browser_session_id = ?', browserSessionId);
    await database_1.default.query('DELETE FROM oauth2_tokens WHERE browser_session_id = ?', browserSessionId);
}
exports.invalidateTokensByBrowserSessionId = invalidateTokensByBrowserSessionId;
function getTokenExpiry() {
    return {
        accessToken: server_settings_1.getSetting('oauth2.accessToken.expiry', 600),
        refreshToken: server_settings_1.getSetting('oauth2.refreshToken.expiry', 3600 * 6),
        code: server_settings_1.getSetting('oauth2.code.expiry', 600),
    };
}
//# sourceMappingURL=service.js.map