"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPathName = exports.recordToModel = exports.save = exports.findByHref = exports.findByIdentity = exports.hasPrincipals = exports.findActiveById = exports.findById = exports.getPrincipalStats = exports.findAll = exports.fieldNames = exports.InactivePrincipal = void 0;
const http_errors_1 = require("@curveball/http-errors");
const database_1 = require("../database");
class InactivePrincipal extends Error {
}
exports.InactivePrincipal = InactivePrincipal;
exports.fieldNames = [
    'id',
    'identity',
    'nickname',
    'created_at',
    'modified_at',
    'type',
    'active'
];
async function findAll(type) {
    let result;
    if (type) {
        const query = `SELECT ${exports.fieldNames.join(', ')} FROM principals WHERE type = ?`;
        result = await database_1.default.query(query, [userTypeToInt(type)]);
    }
    else {
        const query = `SELECT ${exports.fieldNames.join(', ')} FROM principals`;
        result = await database_1.default.query(query);
    }
    const principals = [];
    for (const principal of result[0]) {
        principals.push(recordToModel(principal));
    }
    return principals;
}
exports.findAll = findAll;
async function getPrincipalStats() {
    const query = 'SELECT type, COUNT(*) as total FROM principals GROUP BY type';
    const result = await database_1.default.query(query);
    const principalStats = {
        user: 0,
        app: 0,
        group: 0
    };
    for (const principal of result[0]) {
        principalStats[userTypeIntToUserType(principal.type)] = principal.total;
    }
    return principalStats;
}
exports.getPrincipalStats = getPrincipalStats;
async function findById(id, type) {
    const query = `SELECT ${exports.fieldNames.join(', ')} FROM principals WHERE id = ?`;
    const result = await database_1.default.query(query, [id]);
    if (result[0].length !== 1) {
        throw new http_errors_1.NotFound(`Principal with id: ${id} not found`);
    }
    const principal = recordToModel(result[0][0]);
    if (type && principal.type !== type) {
        throw new http_errors_1.NotFound(`Principal with id ${id} does not have type ${type}`);
    }
    return principal;
}
exports.findById = findById;
async function findActiveById(id) {
    const user = await findById(id);
    if (!user.active) {
        throw new InactivePrincipal(`Principal with identity ${user.identity} is not active`);
    }
    return user;
}
exports.findActiveById = findActiveById;
/**
 * Returns true if more than 1 principal exists in the system.
 *
 * If there are 0 principals, this puts a12nserver in setup mode, which allows a
 * person to create the first user in the system, automatically activate it
 * and make them an admin.
 */
async function hasPrincipals() {
    const query = 'SELECT 1 FROM principals LIMIT 1';
    const result = await database_1.default.query(query);
    return result[0].length > 0;
}
exports.hasPrincipals = hasPrincipals;
async function findByIdentity(identity) {
    const query = `SELECT ${exports.fieldNames.join(', ')} FROM principals WHERE identity = ?`;
    const result = await database_1.default.query(query, [identity]);
    if (result[0].length !== 1) {
        throw new http_errors_1.NotFound(`Principal with identity: ${identity} not found`);
    }
    return recordToModel(result[0][0]);
}
exports.findByIdentity = findByIdentity;
/**
 * Finds a principal by its href'.
 *
 * This can be a string like /user/1, or a full url.
 * It can also be the uri listed in the 'identity' field.
 */
async function findByHref(href) {
    const pathName = getPathName(href);
    const matches = pathName.match(/^\/(user|app|group)\/([0-9]+)$/);
    if (!matches) {
        return findByIdentity(href);
    }
    let typeFilter;
    switch (matches[1]) {
        case 'user':
            // Backwards compatibility
            typeFilter = [1, 2, 3];
            break;
        case 'app':
            typeFilter = [2];
            break;
        case 'group':
            // Backwards compatibility
            typeFilter = [3];
            break;
    }
    const query = `SELECT ${exports.fieldNames.join(', ')} FROM principals WHERE type IN (?) AND id = ?`;
    const result = await database_1.default.query(query, [typeFilter, matches[2]]);
    if (result[0].length !== 1) {
        throw new http_errors_1.NotFound('Principal with href: ' + href + ' not found');
    }
    return recordToModel(result[0][0]);
}
exports.findByHref = findByHref;
async function save(principal) {
    if (!isExistingPrincipal(principal)) {
        // New principal
        const query = 'INSERT INTO principals SET ?';
        const newPrincipalRecord = {
            identity: principal.identity,
            nickname: principal.nickname,
            type: userTypeToInt(principal.type),
            active: principal.active ? 1 : 0,
            modified_at: Date.now(),
            created_at: Date.now(),
        };
        const result = await database_1.default.query(query, [newPrincipalRecord]);
        /* eslint-disable-next-line @typescript-eslint/consistent-type-assertions */
        return ({
            id: result[0].insertId,
            ...principal
        });
    }
    else {
        // Update user
        const query = 'UPDATE principals SET ? WHERE id = ?';
        principal.modifiedAt = new Date();
        const updatePrincipalRecord = {
            identity: principal.identity,
            nickname: principal.nickname,
            active: principal.active ? 1 : 0,
            modified_at: principal.modifiedAt.getTime(),
        };
        await database_1.default.query(query, [updatePrincipalRecord, principal.id]);
        return principal;
    }
}
exports.save = save;
function userTypeIntToUserType(input) {
    switch (input) {
        case 1: return 'user';
        case 2: return 'app';
        case 3: return 'group';
        default:
            throw new Error('Unknown user type id: ' + input);
    }
}
function userTypeToInt(input) {
    switch (input) {
        case 'user': return 1;
        case 'app': return 2;
        case 'group': return 3;
    }
}
function recordToModel(user) {
    return {
        id: user.id,
        href: `/${userTypeIntToUserType(user.type)}/${user.id}`,
        identity: user.identity,
        nickname: user.nickname,
        createdAt: new Date(user.created_at),
        modifiedAt: new Date(user.modified_at),
        type: userTypeIntToUserType(user.type),
        active: !!user.active
    };
}
exports.recordToModel = recordToModel;
function isExistingPrincipal(user) {
    return user.id !== undefined;
}
function getPathName(href) {
    let url;
    try {
        url = new URL(href);
    }
    catch {
        return href;
    }
    return url.pathname;
}
exports.getPathName = getPathName;
//# sourceMappingURL=service.js.map