"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addPrivilegeForUser = exports.replacePrivilegeForUser = exports.findPrivilege = exports.findPrivileges = exports.hasPrivilege = exports.getImmediatePrivilegesForPrincipal = exports.getPrivilegesForPrincipal = void 0;
const database_1 = require("../database");
const database_2 = require("../database");
async function getPrivilegesForPrincipal(principal) {
    const recursiveGroupIds = await getRecursiveGroupIds(principal.id);
    const query = 'SELECT resource, privilege FROM user_privileges WHERE user_id IN (?)';
    const result = await database_1.default.query(query, [recursiveGroupIds]);
    return result[0].reduce((currentPrivileges, row) => {
        const privileges = Object.assign({}, currentPrivileges);
        // eslint-disable-next-line no-prototype-builtins
        if (privileges.hasOwnProperty(row.resource)) {
            if (privileges[row.resource].indexOf(row.privilege) === -1) {
                privileges[row.resource].push(row.privilege);
            }
        }
        else {
            privileges[row.resource] = [row.privilege];
        }
        return privileges;
    }, {});
}
exports.getPrivilegesForPrincipal = getPrivilegesForPrincipal;
async function getImmediatePrivilegesForPrincipal(principal) {
    const query = 'SELECT resource, privilege FROM user_privileges WHERE user_id = ?';
    const result = await database_1.default.query(query, [principal.id]);
    return result[0].reduce((currentPrivileges, row) => {
        const privileges = Object.assign({}, currentPrivileges);
        // eslint-disable-next-line no-prototype-builtins
        if (privileges.hasOwnProperty(row.resource)) {
            if (privileges[row.resource].indexOf(row.privilege) === -1) {
                privileges[row.resource].push(row.privilege);
            }
        }
        else {
            privileges[row.resource] = [row.privilege];
        }
        return privileges;
    }, {});
}
exports.getImmediatePrivilegesForPrincipal = getImmediatePrivilegesForPrincipal;
async function hasPrivilege(who, privilege, resource = '*') {
    var _a, _b;
    let user;
    if (isContext(who)) {
        if (!who.state.user) {
            throw new Error('Cannot check privilege for unauthenticated user');
        }
        user = who.state.user;
    }
    else {
        user = who;
    }
    const privileges = await getPrivilegesForPrincipal(user);
    return ((_a = privileges['*']) === null || _a === void 0 ? void 0 : _a.includes(privilege)) || ((_b = privileges[resource]) === null || _b === void 0 ? void 0 : _b.includes(privilege)) || false;
}
exports.hasPrivilege = hasPrivilege;
function isContext(input) {
    return input.request !== undefined && input.response !== undefined;
}
/**
 * Returns the list of 'privilege types'
 */
async function findPrivileges() {
    const query = `
  SELECT privileges.privilege, privileges.description FROM privileges
  UNION ALL
  SELECT DISTINCT user_privileges.privilege, null FROM user_privileges
  LEFT JOIN privileges
  ON privileges.privilege = user_privileges.privilege
  WHERE privileges.privilege IS NULL
  `;
    const result = await database_2.default.query(query);
    return result[0];
}
exports.findPrivileges = findPrivileges;
async function findPrivilege(privilege) {
    const query = 'SELECT privilege, description FROM privileges WHERE privilege = ?';
    const result = await database_2.default.query(query, [privilege]);
    if (result[0].length !== 1) {
        return {
            privilege: privilege,
            description: ''
        };
    }
    return result[0][0];
}
exports.findPrivilege = findPrivilege;
async function replacePrivilegeForUser(user, privilegeMap) {
    const conn = await database_2.default.getConnection();
    await conn.beginTransaction();
    await conn.query('DELETE FROM user_privileges WHERE user_id = ?', [user.id]);
    for (const [resource, privileges] of Object.entries(privilegeMap)) {
        for (const privilege of privileges) {
            await conn.query('INSERT INTO user_privileges SET ?', [
                {
                    user_id: user.id,
                    privilege,
                    resource,
                }
            ]);
        }
    }
    await conn.commit();
}
exports.replacePrivilegeForUser = replacePrivilegeForUser;
async function addPrivilegeForUser(user, privilege, resource) {
    const query = 'INSERT INTO user_privileges SET ?';
    await database_2.default.query(query, [
        {
            user_id: user.id,
            privilege,
            resource,
        }
    ]);
}
exports.addPrivilegeForUser = addPrivilegeForUser;
/**
 * Returns a list of all group principals for a user.
 *
 * This basically returns a list of ids of all groups that a user is a part of,
 * recursively, and also includes the id of the user itself.
 */
async function getRecursiveGroupIds(principalId) {
    const query = 'SELECT group_id FROM group_members WHERE user_id = ?';
    const result = (await database_2.default.query(query, [principalId]))[0];
    const ids = [
        principalId
    ];
    for (const row of result) {
        ids.push(...await getRecursiveGroupIds(row.group_id));
    }
    return ids;
}
//# sourceMappingURL=service.js.map