"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.editPrivileges = exports.edit = exports.item = exports.collection = void 0;
function collection(users) {
    const hal = {
        _links: {
            'self': { href: '/user' },
            'item': users.map(user => ({
                href: user.href,
                title: user.nickname,
            })),
            'create-form': { href: '/create-user', title: 'Create New User' },
            'find-by-href': {
                title: 'Find a user through a identity/href (exact match)',
                href: '/user/byhref/{href}',
                templated: true,
            },
        },
        total: users.length
    };
    return hal;
}
exports.collection = collection;
/**
 * Generate a HAL response for a specific user
 *
 * hasControl should only be true if the *current* user is same as the user
 * we're generating the repsonse for, or if the current authenticated user
 * has full admin privileges
 */
function item(user, privileges, hasControl, hasPassword, isAdmin, groups) {
    const hal = {
        _links: {
            'self': { href: user.href, title: user.nickname },
            'me': { href: user.identity, title: user.nickname },
            'auth-log': { href: `${user.href}/log`, title: 'Authentication log', type: 'text/csv' },
            'up': { href: '/user', title: 'List of users' },
            'group': groups.map(group => ({
                href: group.href,
                title: group.nickname,
            })),
        },
        nickname: user.nickname,
        active: user.active,
        createdAt: user.createdAt,
        modifiedAt: user.modifiedAt,
        type: user.type,
        privileges
    };
    if (hasControl) {
        hal.hasPassword = hasPassword;
        hal._links['one-time-token'] = {
            href: `/user/${user.id}/one-time-token`,
            title: 'Generate a one-time login token.',
            hints: {
                allow: ['POST'],
            }
        };
        hal._links['access-token'] = {
            href: `/user/${user.id}/access-token`,
            title: 'Generate an access token for this user.',
        };
        hal._links['active-sessions'] = {
            href: `/user/${user.id}/sessions`,
            title: 'Active user sessions'
        };
    }
    if (isAdmin) {
        hal._links['password'] = {
            href: `/user/${user.id}/password`,
            title: 'Change user\'s password',
            hints: {
                allow: ['PUT'],
            }
        };
        hal._links['edit-form'] = {
            href: `/user/${user.id}/edit`,
            title: `Edit ${user.nickname}`
        };
        hal._links['privileges'] = {
            href: `/user/${user.id}/edit/privileges`,
            title: 'Change privilege policy',
        };
    }
    return hal;
}
exports.item = item;
function edit(user) {
    return {
        _links: {
            self: {
                href: `${user.href}/edit`,
            },
            up: {
                href: user.href,
                title: 'Cancel',
            },
        },
        _templates: {
            default: {
                title: 'Edit User',
                method: 'POST',
                contentType: 'application/x-www-form-urlencoded',
                properties: [
                    {
                        name: 'identity',
                        prompt: 'Identity',
                        type: 'text',
                        value: user.identity,
                    },
                    {
                        name: 'nickname',
                        prompt: 'Nickname',
                        type: 'text',
                        value: user.nickname,
                    },
                    {
                        name: 'active',
                        prompt: 'Active',
                        options: {
                            inline: [
                                { prompt: 'Activated', value: 'true' },
                                { prompt: 'Deactivated', value: 'false' },
                            ],
                        },
                        value: user.active ? 'true' : 'false',
                    },
                ],
            },
        },
    };
}
exports.edit = edit;
function editPrivileges(user, privileges) {
    return {
        _links: {
            self: {
                href: `${user.href}/edit/privileges`,
            },
            up: {
                href: user.href,
                title: 'Cancel',
            },
        },
        _templates: {
            default: {
                title: `Edit privilege policy for ${user.nickname}`,
                method: 'POST',
                contentType: 'application/x-www-form-urlencoded',
                properties: [
                    {
                        name: 'policyBody',
                        prompt: 'Privilege Policy',
                        type: 'textarea',
                        cols: 100,
                        rows: 20,
                        value: JSON.stringify(privileges, undefined, 2),
                    },
                ],
            },
        },
    };
}
exports.editPrivileges = editPrivileges;
//# sourceMappingURL=hal.js.map